/*
 * $Id: atacmds.h 588231 2013-06-23 08:02:33Z dqiao $
 *
 * Copyright(C) 1999-2000 Michael Cornwell <cornwell@acm.org>
 * Copyright(C) 2000 Andre Hedrick <andre@linux-ide.org>
 *
 * Copyright (c) 2002-2012, Juniper Networks, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * You should have received a copy of the GNU General Public License
 *(for example COPYING); if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef _ATACMDS_H_
#define _ATACMDS_H_

#include <sys/ata-junos.h>

/* These defines SHOULD BE in the kernel
   if not we define them */

#ifndef WIN_SMART
#define WIN_SMART		0xb0	
#endif

#ifndef SMART_READ_VALUES
#define SMART_READ_VALUES	0xd0
#endif

#ifndef SMART_READ_THRESHOLDS	
#define SMART_READ_THRESHOLDS	0xd1
#endif

#ifndef  SMART_AUTOSAVE 
#define SMART_AUTOSAVE		0xd2
#endif

#ifndef SMART_SAVE
#define SMART_SAVE		0xd3
#endif

#ifndef SMART_IMMEDIATE_OFFLINE	
#define SMART_IMMEDIATE_OFFLINE	0xd4
#endif

#ifndef SMART_READ_LOG_SECTOR
#define SMART_READ_LOG_SECTOR 0xd5
#endif

#ifndef SMART_WRITE_LOG_SECTOR
#define SMART_WRITE_LOG_SECTOR 0xd6
#endif

#ifndef SMART_ENABLE
#define SMART_ENABLE		0xd8
#endif

#ifndef SMART_DISABLE
#define SMART_DISABLE		0xd9
#endif

#ifndef SMART_STATUS
#define SMART_STATUS		0xda
#endif

#ifndef SMART_AUTO_OFFLINE
#define SMART_AUTO_OFFLINE	0xdb
#endif

/* The value to be passed to the drive is in multiples of 5 seconds
 * Disk's standby inactivity timer is set to 10 minutes
 *
 * (10*60/5) = 120
 */
#define     AD_INACTIVITY_TIME_PARAM    120

#define ATA_POWERMODE_STANDBY	0x00
#define ATA_POWERMODE_IDLE	0x80
#define ATA_POWERMODE_ACTIVE_OR_IDLE	0xff

/* Self test types */
#define OFFLINE_FULL_SCAN			0
#define SHORT_SELF_TEST				1
#define EXTEND_SELF_TEST			2
#define SHORT_CAPTIVE_SELF_TEST		129
#define EXTEND_CAPTIVE_SELF_TEST	130

/* ATA commands for formatting */
#define SEC_ERASE_PREPARE	0xf3
#define FORMAT_UNIT		0xf7

/* Abort bit in the error register */
#define ATA_ABORT        0x4

#define NUMBER_ATA_SMART_ATTRIBUTES 30

/* Smart Values Data Structures */

/* ata_smart_attribute is the vendor specific in SFF-8035 spec */ 
struct ata_smart_attribute {
	unsigned char id;
	unsigned short status_flag;
	/* note char array used since linux can 
       not handle sizeof short correctly */
	unsigned char normalized;
	unsigned char worse_normal;
	unsigned char raw[6];
	unsigned char reserv;
} __attribute__((packed));

/* ata_smart_values is format of the read drive Atrribute command */
struct ata_smart_values {
	unsigned short int        revnumber;
	struct ata_smart_attribute    vendor_attributes [NUMBER_ATA_SMART_ATTRIBUTES];
	unsigned char             offline_data_collection_status;
	unsigned char             self_test_exec_status;
	unsigned short int        total_time_to_complete_off_line;
	unsigned char             vendor_specific_366;
	unsigned char             offline_data_collection_capability;
	unsigned short int        smart_capability;
	unsigned char             errorlog_capability;
	unsigned char             vendor_specific_371;
	unsigned char             short_test_completion_time;
	unsigned char             extend_test_completion_time;
	unsigned char			  reserved_374_385 [12];
	unsigned char             vendor_specific_386_509 [125];
	unsigned char             chksum;
} __attribute__((packed));

/* Smart Threshold data structures */
/* Vendor attribute of SMART Threshold */
struct ata_smart_threshold_entry {
	unsigned char id;
	unsigned char normalized_threshold;
	unsigned char reserved[10];
} __attribute__((packed));

/* Format of Read SMART THreshold Command */
struct ata_smart_thresholds {
	unsigned short int revnumber;
	struct ata_smart_threshold_entry thres_entries[30];
	unsigned char reserved[149];
	unsigned char chksum;
} __attribute__((packed));

struct ata_smart_errorlog_command_struct {
	unsigned char devicecontrolreg;
	unsigned char featuresreg;
	unsigned char sector_count;
	unsigned char sector_number;
	unsigned char cylinder_low;
	unsigned char cylinder_high;
	unsigned char drive_head;
	unsigned char commandreg;
	unsigned int timestamp; /*Really and int but Linux Size of is broken*/
} __attribute__((packed));

struct ata_smart_errorlog_error_struct {
	unsigned char error_condition;
	unsigned char extended_error[14];
	unsigned char state;
	unsigned short timestamp; /* Fucked up linux sizeof */
} __attribute__((packed));

struct ata_smart_errorlog_struct {
	struct ata_smart_errorlog_command_struct commands[6];
	struct ata_smart_errorlog_error_struct error_struct;
}  __attribute__((packed));

struct ata_smart_errorlog {
	unsigned char revnumber;
	unsigned char error_log_pointer;
	struct ata_smart_errorlog_struct errorlog_struct[5];
	unsigned short int ata_error_count;
	unsigned short int non_fatal_count;
	unsigned short int drive_timeout_count;
	unsigned char reserved[53];
} __attribute__((packed));

struct ata_smart_selftestlog_struct {
	unsigned char selftestnumber;
	unsigned char selfteststatus;
	unsigned short int timestamp;
	unsigned char selftestfailurecheckpoint;
	unsigned int lbafirstfailure;
	unsigned char vendorspecific[15];
} __attribute__((packed));

struct ata_smart_selftestlog {
	unsigned short int revnumber;
	struct ata_smart_selftestlog_struct selftest_struct[21];
	unsigned char vendorspecific[2];
	unsigned char mostrecenttest;
	unsigned char resevered[2];
	unsigned char chksum;
} __attribute__((packed));

/* Read SMART information from drive */
int atareadhdidentity(int device, struct ata_params *buf);
int atareadsmartvalues(int device,struct ata_smart_values *);
int atareadsmartthresholds(int device, struct ata_smart_thresholds *);
int atareaderrorlog(int device, struct ata_smart_errorlog *);
int atareadselftestlog(int device, struct ata_smart_selftestlog *);
int atasmartstatus(int device);
int atasmarttest(int device, int testtype);
int atasetsmartthresholds(int device, struct ata_smart_thresholds *);

/* Enable/Disable SMART on device */
int ataenablesmart(int device );
int atadisablesmart(int device );

/* Automatic Offline Testing */
int ataenableautooffline(int device );
int atadisableautooffline(int device );

/* SMART test commands */
int atasmartofflinetest(int device);
int atasmartextendselftest(int device);
int atasmartshortselftest(int device);
int atasmartshortcapselftest(int device);
int atasmartextendcapselftest(int device);
int atasmartselftestabort(int device);
int atacheckdevicestatus (int device, int *error);

/* Format unit */
int ataformatunit(int device, struct ata_params *drive);

/*Check Parameters of Smart Data */

int ataversioninfo(struct ata_params *drive);

/*  int atasmartsupport(struct ata_params *drive)
*   Check if SMART is supported and enabled in drive 
*	returns -1:if SMART cabability can not be checked
*	returns	0: if drive does not support SMART
*			1: if drive supports SMART but not enabled 
*			2: if drive supports SMART and enabled 
*		  255: if drive supports SMART but does not   
*			   support ATA-4. 
*	ATA 3 and lower do not support SMART enabled bit
*   Attempt a Read SMART attributes to check if enabled
*/ 
int atasmartsupport(struct ata_params *drive);

/* Check SMART for Threshold failure */
int atachecksmart(struct ata_smart_values data, struct ata_smart_thresholds thresholds);
/* Issue Standby command to drive */
int ataissuestandby (int device, unsigned int timeinterval);

/* Issue Standby Immediate command to drive */
int ataissuestandbyimmediate (int device);

/* Return current disk power mode */
char atacheckpowermode (int device, unsigned char powercheckenabled);

/* Check whether Power Management set feautres are supported */
int atapwrmgmtsupport (struct ata_params *drive);

/* Print disk capabilities */
void cap_print (struct ata_params *drive);

int atastandbyarmset(int device, char arm_standby);
char atastandbyarmget(int device, ad_arm_t *arm_info);
int atawritecacheset (int device, char enable);
int ataissueidle (int device, unsigned int timeinterval);

/* int isofflinetesttime(struct ata_smart_values data)
 * returns SMART Offline Test Time in seconds
 */
int isofflinetesttime(struct ata_smart_values data);

int isshortselftesttime(struct ata_smart_values data);
int isextendedselftesttime(struct ata_smart_values data);
int issmarterrorlogcapable(struct ata_smart_values data);
int issupportexecuteofflineimmediate(struct ata_smart_values data);
int issupportautomatictimer(struct ata_smart_values data);
int issupportofflineabort(struct ata_smart_values data);
int issupportofflinesurfacescan(struct ata_smart_values data);
int issupportselftest(struct ata_smart_values data);

#endif /* _ATACMDS_H_ */
