/*
 * $Id: version_gpl.h 783309 2016-05-29 20:43:57Z ib-builder $
 *
 * version_gpl.h -- version structure
 *
 * Copyright (c) 2002-2012, Juniper Networks, Inc.
 * All rights reserved.
 */

#ifndef VERSION_GPL_H
#define VERSION_GPL_H

#define SWV_WHAT_MAGIC	"@(#)"	/* Tag that what(1) searches for */
#define SWV_JNX_MAGIC	"@JNX"	/* Tag that tells us our header is present */

enum sw_category_enum {
    SWV_CAT_UNKNOWN = 0,	/* Unknown value; error */
    SWV_CAT_ALPHA = 'a',	/* Official Alpha build */
    SWV_CAT_BETA = 'b',		/* Official Beta build */
    SWV_CAT_RELEASE = 'r',	/* Official Release build ("Rah!") */
    SWV_CAT_PATCH = 'p',	/* Unofficial patch / bug fix */
    SWV_CAT_INTERNAL = 'i',	/* Internal build (as a release) */
    SWV_CAT_DEVELOPER = 'n'	/* Not a release build, just a normal make */
};

typedef struct sw_version_s {
    char sw_jnx_magic[ 4 ];	/* SWV_JNX_MAGIC */
    unsigned char sw_category;	/* Build category (SWV_CAT_*) */
    unsigned char sw_major;	/* Major release number */
    unsigned short sw_minor;	/* Minor release number */
    unsigned short sw_build;	/* Build number */
    char sw_version[ 128 ];	/* Printable version number */
    char sw_tag[ 32 ];		/* Build tag (optional description) */
    char sw_builder[ 16 ];	/* Builder's login */
    char sw_date[ 32 ];		/* Date of build in ISO format */
    char sw_what_magic[ 4 ];	/* SWV_WHAT_MAGIC */
    char sw_what_line[ 256 ];	/* Output from the what command */
    char sw_fancy[ 512 ];	/* Fancy version */
    char sw_what_magic2[ 4 ];	/* SWV_WHAT_MAGIC */
    char sw_build_info[ 512 ];	/* $hostname:$pwd of builder */
    char sw_build_machine[ 32 ];	/* Builder's machine name */
    char sw_build_directory[ 256 ];	/* Build Directory */
    char sw_component_name[ 64 ];	/* Component (daemon) name */
    unsigned short sw_spin;	/* Spin value */
} sw_version_t;

#ifndef VERSION_C
sw_version_t sw_version; /* This module's version information */

static inline const char *
swversion_what (void) {
    return sw_version.sw_what_line;
}

static inline const char *
swversion_build_info (void) {
    return sw_version.sw_build_info;
}

#else /* VERSION_C */
extern volatile sw_version_t sw_version; /* This module's version information */
#endif /* VERSION_C */
#endif /* VERSION_GPL_H */
